#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>

#include "images.h"

// lisant le fichier filename et écrivant son contenu dans la variable p. Cette fonction retourne un pgm_error.
pgm_error pgm_read_from_file(pgm *p, char *filename){
     // open a file in mode binary
     FILE *baseImg = fopen(filename, "rb"); // read binary

     if (baseImg == NULL) {
          fprintf(stderr, "Can't open output file %s!\n", filename); // show error
          return failure;
     }

     char buffer[100];
     fgets(buffer, 37, baseImg); // read the first line

     if (strcmp(buffer, "P5\n") != 0) {
          return failure;
     }

     fgets(buffer, 37, baseImg); //read the next line
     char *t = strtok(buffer, " ");
     int width = atoi(t); //convert string to int
     t = strtok(NULL, " ");
     int height = atoi(t);

     fgets(buffer, 37, baseImg); //read the next line
     p->max = atoi(buffer);

     int32_t bufferB[width * height];
     for (int32_t i = 0; i < width * height; i++) {
          uint8_t tempPixel;
          fread(&(tempPixel), sizeof(uint8_t), 1, baseImg);
          bufferB[i] = (int32_t)tempPixel;
     }

     if (matrix_init_from_array(&p->pixels, height, width, bufferB, height * width) != ok) {
          return failure;
     }

     fclose(baseImg);

     return success;

}

// écrivant le contenu de l’image p dans le le fichier filename. Cette fonction retourne un pgm_error.
pgm_error pgm_write_to_file(pgm *p, char *filename){
     // open a file in mode binary
     FILE *baseImg = fopen(filename, "w+b"); // write binary
     if (baseImg == NULL) {
          fprintf(stderr, "Can't open output file %s!\n", filename); // show error
          return failure;
     }

     fprintf(baseImg, "P5\n%d %d\n%d\n", p->pixels.n, p->pixels.m, p->max); // write the first 3 lines

     for (int32_t i = 0; i < p->pixels.m; i++) {
          for (int32_t j = 0; j < p->pixels.n; j++) {
               fwrite(&p->pixels.data[i][j], 1, 1, baseImg);
          }
     }

     fclose(baseImg);

     return success;
}

pgm_error pgm_destroy(pgm *pgm){
     if(matrix_destroy(&pgm->pixels) != ok) {
          return failure;
     }

     free(pgm);

     return ok;
}

// calculant le négatif de l’image orig et la stockant dans neg (qui est également allouée dans cette fonction). L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
pgm_error pgm_negative(pgm *neg, const pgm *const orig){
     neg->max = orig->max;

     if(matrix_alloc(&neg->pixels, orig->pixels.m, orig->pixels.n) != ok) {
          return failure;
     }

     for (int32_t i = 0; i < orig->pixels.n; i++) {
          for (int32_t j = 0; j < orig->pixels.m; j++) {
               neg->pixels.data[i][j] = neg->max - 1 - orig->pixels.data[i][j];
          }
     }

     return success;
}

// calculant la symétrie horizontale de l’image orig et la stockant dans sym (qui est également allouée dans cette fonction). L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
pgm_error pgm_symmetry_hori(pgm *sym, const pgm *const orig){
     sym->max = orig->max;

     if(matrix_alloc(&sym->pixels, orig->pixels.m, orig->pixels.n) != ok){
          return failure;
     }

     for (int32_t i = 0; i < orig->pixels.n; i++) {
          for (int32_t j = 0; j < orig->pixels.m; j++) {
               sym->pixels.data[i][j] = orig->pixels.data[i][orig->pixels.n - 1 - j];
          }
     }

     return success;
}

// calculant la symétrie verticale de l’image orig et la stockant dans sym (qui est également allouée dans cette fonction). L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
pgm_error pgm_symmetry_vert(pgm *sym, const pgm *const orig){
     sym->max = orig->max;

     if(matrix_alloc(&sym->pixels, orig->pixels.m, orig->pixels.n) != ok){
          return failure;
     }

     for (int32_t i = 0; i < orig->pixels.n; i++) {
          for (int32_t j = 0; j < orig->pixels.m; j++) {
               sym->pixels.data[i][j] = orig->pixels.data[orig->pixels.m - 1 - i][j];
          }
     }

     return success;
}

// calculant la symétrie centrale de l’image orig et la stockant dans sym (qui est également allouée dans cette fonction). L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
pgm_error pgm_symmetry_cent(pgm *sym, const pgm *const orig){
     sym->max = orig->max;

     if(matrix_alloc(&sym->pixels, orig->pixels.m, orig->pixels.n) != ok){
          return failure;
     }

     for (int32_t i = 0; i < orig->pixels.n; i++) {
          for (int32_t j = 0; j < orig->pixels.m; j++) {
               sym->pixels.data[i][j] = orig->pixels.data[orig->pixels.m - 1 - i][orig->pixels.n - 1 - j];
          }
     }

     return success;
}

// calculant l’effet photomaton de l’image orig et la stockant dans photomaton (qui est également allouée dans cette fonction). L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
pgm_error pgm_photomaton(pgm *photomaton, const pgm *const orig){
     photomaton->max = orig->max;

     if(matrix_alloc(&photomaton->pixels, orig->pixels.m, orig->pixels.n) != ok){
          return failure;
     }

     for (int32_t i = 0; i < orig->pixels.n; i++) {
          for (int32_t j = 0; j < orig->pixels.m; j++) {
               photomaton->pixels.data[(orig->pixels.n / 2) * (i % 2) + (i / 2)][(orig->pixels.m / 2) * (j % 2) + (j / 2)] = orig->pixels.data[i][j];
          }
     }

     return success;
}

// calculant la sous-image de orig aux coordonnées x0 à x1 (non-inclu), et y0 à y1 (non-inclu).
// Le résultat est stocké dans crop (qui est également allouée dans cette fonction).
// L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
// extraction d’une sous-matrice d’une matrice
// error_code matrix_extract_submatrix(matrix *sub, const matrix mat, int32_t m0, int32_t m1, int32_t n0, int32_t n1);
pgm_error pgm_crop(pgm *crop, const pgm *const orig, int32_t x0, int32_t x1, int32_t y0, int32_t y1){
     crop->max = orig->max;

     if(matrix_extract_submatrix(&crop->pixels, orig->pixels, x0, x1, y0, y1) != ok){
          return failure;
     }

     return success;
}

// calculant la convolution entre orig et le noyau kernel. Le résultat est stocké dans conv (qui est également allouée dans cette fonction).
// L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
pgm_error pgm_conv(pgm *conv, const pgm *const orig, const matrix *const kernel, double norm){
     conv->max = orig->max;

     if(matrix_alloc(&conv->pixels, orig->pixels.m, orig->pixels.n) != ok){
          return failure;
     }

     int32_t h = (kernel->n -1) / 2;
     int32_t g = (kernel->m -1) / 2;

     for (int32_t i = 0; i < orig->pixels.n; i++) {
          for (int32_t j = 0; j < orig->pixels.m; j++) {
               int32_t resConv = 0;
               for (int32_t k = 0; k < kernel->n; k++) {
                    for (int32_t l = 0; l < kernel->m; l++) {
                         int32_t m = j - g - l;
                         int32_t n = i - h - k;

                         //check if the pixel is inside the matrix of the image or not
                         if (!(m < 0 || n < 0 || m >= orig->pixels.m || n >= orig->pixels.n)) {
                              //get the pixel of the orig
                              resConv += orig->pixels.data[n][m] * kernel->data[k][l];
                         }
                    }
               }
               resConv *= norm;
               if (resConv < 0) {
                    resConv = 0;
               }

               if (resConv > orig->max - 1) {
                    resConv = orig->max - 1;
               }
               conv->pixels.data[i][j] = resConv;
          }
     }

     return success;
}
