#ifndef _IMGAGES_H_
#define _IMAGES_H_

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include "matrix.h"

typedef enum _pgm_error {
     success, failure
} pgm_error;

typedef struct _pgm {
     int32_t max;
     matrix pixels;
} pgm;

// lisant le fichier filename et écrivant son contenu dans la variable p. Cette fonction retourne un pgm_error.
pgm_error pgm_read_from_file(pgm *p, char *filename);

//supprime le pgm
pgm_error pgm_destroy(pgm *p);

// écrivant le contenu de l’image p dans le le fichier filename. Cette fonction retourne un pgm_error.
pgm_error pgm_write_to_file(pgm *p, char *filename);

// calculant le négatif de l’image origet la stockant dans neg (qui est également allouée dans cette fonction). L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
pgm_error pgm_negative(pgm *neg, const pgm *const orig);

// calculant la symétrie horizontale de l’image orig et la stockant dans sym (qui est également allouée dans cette fonction). L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
pgm_error pgm_symmetry_hori(pgm *sym, const pgm *const orig);

// calculant la symétrie verticale de l’image orig et la stockant dans sym (qui est également allouée dans cette fonction). L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
pgm_error pgm_symmetry_vert(pgm *sym, const pgm *const orig);

// calculant la symétrie centrale de l’image orig et la stockant dans sym (qui est également allouée dans cette fonction). L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
pgm_error pgm_symmetry_cent(pgm *sym, const pgm *const orig);

// calculant l’effet photomaton de l’image orig et la stockant dans photomaton (qui est également allouée dans cette fonction). L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
pgm_error pgm_photomaton(pgm *photomaton, const pgm *const orig);

// calculant la sous-image de orig aux coordonnées x0 à x1 (non-inclu), et y0 à y1 (non-inclu).
// Le résultat est stocké dans crop (qui est également allouée dans cette fonction).
// L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
pgm_error pgm_crop(pgm *crop, const pgm *const orig, int32_t x0, int32_t x1, int32_t y0, int32_t y1);


// calculant la convolution entre orig et le noyau kernel. Le résultat est stocké dans conv (qui est également allouée dans cette fonction).
// L’image orig n’est pas modifiée. Cette fonction retourne un pgm_error.
pgm_error pgm_conv(pgm *conv, const pgm *const orig, const matrix *const kernel, double norm);

#endif
