#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>

#include "images.h"
#include "matrix.h"
#include "gfx.h"

static void render(struct gfx_context_t *context, pgm *p) {
     gfx_clear(context, COLOR_BLACK);

     for (int32_t i = 0; i < p->pixels.m; i++) {
          for (int32_t j = 0; j < p->pixels.n; j++) {
               uint32_t intensity = p->pixels.data[i][j];
               uint32_t color = MAKE_COLOR(intensity,intensity,intensity);
               gfx_putpixel(context, j, i, color);
          }
     }
}

int main(){
     pgm *p = malloc(sizeof(pgm));
     pgm *neg = malloc(sizeof(pgm));
     pgm *hori = malloc(sizeof(pgm));
     pgm *verti = malloc(sizeof(pgm));
     pgm *centra = malloc(sizeof(pgm));
     pgm *phantom = malloc(sizeof(pgm));
     pgm *croppppp = malloc(sizeof(pgm));
     pgm *convvvvvv = malloc(sizeof(pgm));
     matrix kernel;

     //Filter convolution
     // int32_t subArray[9] = {-1, -1, -1, -1, 8, -1, -1, -1, -1};
     // int32_t subArray[9] = {0, -1, 0, -1, 5, -1, 0, -1, 0};
     int32_t subArray[25] = {1, 4, 6, 4, 1, 4, 16, 24, 16, 4, 6 ,24, 36, 24, 6, 4, 16, 24, 16, 4, 1, 4, 6, 4, 1};

     // int kernelX = 3;
     // int kernelY = 3;
     int kernelX = 5;
     int kernelY = 5;

     pgm_read_from_file(p, "img/mandrill.pgm");
     pgm_write_to_file(p, "img/clone.pgm");

     pgm_negative(neg, p);
     pgm_write_to_file(neg, "img/neg.pgm");

     pgm_symmetry_hori(hori, p);
     pgm_symmetry_vert(verti, p);
     pgm_symmetry_cent(centra, p);

     pgm_write_to_file(hori, "img/hori.pgm");
     pgm_write_to_file(verti, "img/verti.pgm");
     pgm_write_to_file(centra, "img/centra.pgm");

     pgm_photomaton(phantom, p);
     pgm_write_to_file(phantom, "img/phantom.pgm");

     pgm_crop(croppppp, p, 10, 200, 10 , 200);
     pgm_write_to_file(croppppp, "img/crop.pgm");

     matrix_init_from_array(&kernel, kernelX, kernelY, subArray, kernelY * kernelX);
     pgm_conv(convvvvvv, p, &kernel, 1/256.0);
     pgm_write_to_file(convvvvvv, "img/conv.pgm");

     matrix_destroy(&kernel);

     struct gfx_context_t *ctxt = gfx_create("Image PGM", p->pixels.n, p->pixels.m);
     if (!ctxt) {
          fprintf(stderr, "Graphics initialization failed!\n");
          return EXIT_FAILURE;
     }

     while (gfx_keypressed() != SDLK_ESCAPE) {
          render(ctxt, convvvvvv);
          gfx_present(ctxt);
     }

     gfx_destroy(ctxt);

     pgm_destroy(p);
     pgm_destroy(neg);
     pgm_destroy(hori);
     pgm_destroy(verti);
     pgm_destroy(centra);
     pgm_destroy(phantom);
     pgm_destroy(croppppp);
     pgm_destroy(convvvvvv);
}
